*:*********************************************************************
*:
*:        Program: SORT_IT
*:
*:         System: Sort text files
*:         Author: John Wright
*:      Copyright (c) 1991-1995, John Wright
*:
*:*********************************************************************
* 10/03/91 - Create a program to sort files greater than 64k in size.
*            Tried the DOS SORT command but files got too big.
* 10/07/91 - Put in option to reduce "snow" on CGA screens.  Added more
*            text in description.  Show record numbers to while indexing.
* 03/12/95 - Sort by length of line and/or UPPER case.  Updated text
*            on help screen and cleaned up program code.

#INCLUDE database.hdr
#INCLUDE fileio.hdr
#INCLUDE screen.hdr
#INCLUDE string.hdr
#INCLUDE system.hdr
#INCLUDE io.hdr

#PRAGMA W_FUNC_PROC-

* Define variables used by system
VARDEF
  CHAR      cPattern
  CHAR      cFileName
  CHAR      cText
  CHAR      cTrimText
  FILE      fHandle
  INT       nRow
  INT       nCol
  INT       nLength
  INT       nMarker
  INT       nBlank
  INT       nTrunc
  DBL       nLineNo
ENDDEF

* Simple function used to show records while indexing is taking place.
* This must be defined before the index key in INDEXDEF.
FUNCTION CHAR ndx_rec
@ nRow,nCol ?? RECNO()
RETURN ""
ENDPRO

* Define database structure
DBFDEF tmp_sort
  CHAR(80)  tmp_line
ENDDEF

* Define index keys
INDEXDEF
  CHAR(80) tmp_fdx tmp_sort->tmp_line+NDX_REC()
ENDDEF

INDEXDEF
  CHAR(80) tmp_upr UPPER(tmp_sort->tmp_line)+NDX_REC()
ENDDEF

INDEXDEF
  CHAR(80) tmp_len STR(LEN(RTRIM(tmp_sort->tmp_line)),2,0)+tmp_sort->tmp_line+NDX_REC()
ENDDEF

INDEXDEF
  CHAR(80) tmp_uprlen STR(LEN(RTRIM(tmp_sort->tmp_line)),2,0)+UPPER(tmp_sort->tmp_line)+NDX_REC()
ENDDEF

*!*********************************************************************
*!
*!      Procedure: FORCE_MAIN
*!
*!*********************************************************************
PROCEDURE force_main
PARAMETERS CHAR(127) cmd_line

* Try to eliminate flicker on CGA screens.
DO scrn_dos

cPattern := UPPER(LTRIM(cmd_line))+" "
nLength  := LEN(cPattern)

IF nLength <= 1 .OR. "?" $ cPattern
   ?"Ŀ"
   ?" SORT_IT - 1.2                            Text file sorting utility "
   ?"Ĵ"
   ?" Sort a text file and write results back to the same filename.      "
   ?"                                                                    "
   ?" Maximum column width is 80 characters.  Lines longer than 80 will  "
   ?" be truncated!  Restriction due to method used for sorting files.   "
   ?"                                                                    "
   ?" SORT_IT is written in Xbase and compiled by Force.  It uses a DBF  "
   ?" database file to store and sort records.  It is not limited by RAM "
   ?" memory like the DOS SORT command.  Temporary database and index    "
   ?" files are created when SORT_IT is run.  They are deleted once the  "
   ?" sort is complete.  NOTE: Blank lines are removed from the file.    "
   ?"                                                                    "
   ?" This is a FREE program written by John Wright [70534,2330]         "
   ?"Ĵ"
   ?" Syntax:  SORT_IT <filename> [/L] [/U] [/B] [/T]                    "
   ?"                                                                    "
   ?"          /L sort by length of line                                 "
   ?"          /U sort lines in UPPER case (does not alter actual text)  "
   ?"          /B remove any blank space before text                     "
   ?"          /T continue sorting even if lines will be truncated       "
   ?""
   ?" "
   QUIT
ENDIF

* Grab file name
nMarker   := AT(" ",cPattern)
cFileName := SUBSTR( cPattern, 1, nMarker - 1 )

IF .NOT. EXIST(cFileName)
  ?"File not found => "+cFileName
  ?" "
  QUIT
ELSE
  ?"Sort file: "+cFileName
  ?" "
ENDIF

Cursor_off()

* create the database to use
BUILD "$sort_it.dbf" FROM ALIAS tmp_sort
USE "$sort_it.dbf" ALIAS tmp_sort

* Open text file
F_open( fHandle, cFileName, &F_READ, &F_TEXT )

?"Reading..."
nRow := ROW()
nCol := COL()
nLineNo := 0
nBlank  := 0
nTrunc  := 0

DO WHILE .NOT. F_eof( fHandle )

   nLineNo ++
   @ nRow,nCol ?? nLineNo

   F_getln( fHandle, cText )

   * Trim leading spaces?
   IF AT("/B", cPattern) > 0
      cTrimText := LTRIM( RTRIM( cText ) )
   ELSE
      cTrimText := RTRIM( cText )
   ENDIF

   * Ignore blank lines!
   IF LEN( cTrimText ) > 0
      APPEND BLANK
      REPLACE tmp_sort->tmp_line WITH SUBSTR( cTrimText, 1, 80 )
      * Will the line be truncated?
      IF LEN( cTrimText ) > 80
         nTrunc ++
      ENDIF
   ELSE
      * Ignore "blank" line caused by end-of-file marker
      IF F_eof( fHandle )
         nLineNo --
         @ nRow,nCol ?? nLineNo
      ELSE
         nBlank ++
      ENDIF
   ENDIF
ENDDO

* Close text file
F_close( fHandle )

IF nBlank > 0
   ?"Removed..."
   nRow := ROW()
   nCol := COL()
   @ nRow,nCol ?? nBlank
   ?? " blank line"
   IF nBlank > 1
      ??"s"
   ENDIF
ENDIF

IF nTrunc > 0
   ?"Truncated!"
   nRow := ROW()
   nCol := COL()
   @ nRow,nCol ?? nTrunc
   ?? " long line"
   IF nTrunc > 1
      ??"s"
   ENDIF
ENDIF

* Only sort file if no truncation or truncating is allowed
IF nTrunc = 0 .OR. AT("/T", cPattern) > 0

   * Re-open file for writing
   F_open( fHandle, cFileName, &F_CREATE, &F_TEXT )

   ?"Sorting..."
   nRow := ROW()
   nCol := COL()

   * Index database records
   DO CASE
   CASE AT("/L", cPattern) > 0 .AND. AT("/U", cPattern) > 0
      INDEX "$sort_it.fdx" ALIAS tmp_uprlen
   CASE AT("/L", cPattern) > 0
      INDEX "$sort_it.fdx" ALIAS tmp_len
   CASE AT("/U", cPattern) > 0
      INDEX "$sort_it.fdx" ALIAS tmp_upr
   OTHERWISE
      INDEX "$sort_it.fdx" ALIAS tmp_fdx
   ENDCASE

   ?"Writing..."
   nRow := ROW()
   nCol := COL()
   nLineNo := 0

   GO TOP
   DO WHILE .NOT. EOF()
      nLineNo ++
      @ nRow,nCol ?? nLineNo
      cText := RTRIM( tmp_sort->tmp_line ) + CHR(13) + CHR(10)
      F_put( fHandle, cText )
      SKIP
   ENDDO

   * Close text file
   F_close( fHandle )
ELSE
   ?" "
   ?"*** File NOT changed due to potential text truncation!"
ENDIF

* Close database & index
CLOSE tmp_sort

* Erase temporary database & index
IF EXIST( "$sort_it.dbf" )
   ERASE "$sort_it.dbf"
ENDIF
IF EXIST( "$sort_it.fdx" )
   ERASE "$sort_it.fdx"
ENDIF

?" "
Cursor_on()
QUIT

ENDPRO

*: EOF: SORT_IT.PRG
